@@ -15,6 +15,9 @@ import time
 import io
 import tokenize
 import token
+import logging
+
+logger = logging.getLogger(__name__)
 
 # Template for application setup.py script
 setup = """from setuptools import setup, find_packages
@@ -106,7 +109,7 @@ class ImportsFinder:
 
     def __init__(self, *args, **kw):
         self.package = kw.pop("package") or ""
-        
+
     def find(self, src):
         """Find imports in source code src. Uses the tokenize module instead
         of ast in previous Brython version, so that this script can be run
@@ -202,10 +205,13 @@ class ImportsFinder:
 
 class ModulesFinder:
 
-    def __init__(self, directory=os.getcwd()):
+    def __init__(self, directory=os.getcwd(), stdlib={}, user_modules={}):
         self.directory = directory
         self.modules = set()
 
+        self.stdlib = stdlib
+        self.user_modules = user_modules
+
     def get_imports(self, src, package=None):
         """Get all imports in source code src."""
         finder = ImportsFinder(package=package)
@@ -214,7 +220,7 @@ class ModulesFinder:
             if module in self.modules:
                 continue
             found = False
-            for module_dict in [stdlib, user_modules]:
+            for module_dict in [self.stdlib, self.user_modules]:
                 if module in module_dict:
                     found = True
                     self.modules.add(module)
@@ -291,6 +297,8 @@ class ModulesFinder:
                     if package is not None and \
                             package.startswith(site_packages):
                         package = package[len('Lib/site-packages/'):]
+
+                    # print(path)
                     with open(path, encoding="utf-8") as fobj:
                         try:
                             imports |= self.get_imports(fobj.read(), package)
@@ -298,13 +306,13 @@ class ModulesFinder:
                             print('syntax error', path)
                             traceback.print_exc(file=sys.stderr)
 
-    def make_brython_modules(self):
+    def make_brython_modules(self, path):
         """Build brython_modules.js from the list of modules needed by the
         application.
         """
         vfs = {"$timestamp": int(1000 * time.time())}
         for module in self.modules:
-            dico = stdlib if module in stdlib else user_modules
+            dico = self.stdlib if module in self.stdlib else self.user_modules
             vfs[module] = dico[module]
             elts = module.split('.')
             for i in range(1, len(elts)):
@@ -312,7 +320,35 @@ class ModulesFinder:
                 if not pkg in vfs:
                     vfs[pkg] = dico[pkg]
         # save in brython_modules.js
-        path = os.path.join(stdlib_dir, "brython_modules.js")
+        if os.path.exists(path):
+            # If brython_modules.js already exists, check if there have been
+            # changes. Cf. issue #1471.
+            changes = False
+            with open(path, encoding="utf-8") as f:
+                content = f.read()
+                start_str = "var scripts = "
+                start_pos = content.find(start_str)
+                end_pos = content.find("__BRYTHON__.update_VFS(scripts)")
+                data = content[start_pos + len(start_str):end_pos].strip()
+                old_vfs = json.loads(data)
+                if old_vfs.keys() != vfs.keys():
+                    changes = True
+                else:
+                    changes = True
+                    for key in old_vfs:
+                        if key == "$timestamp":
+                            continue
+                        if not key in vfs:
+                            break
+                        elif vfs[key][1] != old_vfs[key][1]:
+                            break
+                    else: # no break
+                        changes = False
+
+            if not changes:
+                print("No change: brython_modules.js not updated")
+                return
+
         with open(path, "w", encoding="utf-8") as out:
             # Add VFS_timestamp ; used to test if the indexedDB must be
             # refreshed
@@ -435,30 +471,40 @@ class ModulesFinder:
         with open(path, "w", encoding="utf-8") as out:
             out.write(app.format(**info))
 
-# User-defined Python modules and packages
-user_modules = {}
 
 # Get all modules in the Brython standard distribution.
 # They must be in brython_stdlib.js somewhere in the current directory
 # or below.
-print('searching brython_stdlib.js...')
-stdlib = {}
-stdlib_dir = None
-for dirname, dirnames, filenames in os.walk(os.getcwd()):
-    for filename in filenames:
-        if filename == "brython_stdlib.js":
-            stdlib_dir = dirname
-            path = os.path.join(dirname, filename)
-            with open(path, encoding="utf-8") as fobj:
-                modules = fobj.read()
-                modules = modules[modules.find('{'):
-                        modules.find('__BRYTHON__.update_VFS(')]
-                stdlib = json.loads(modules)
+def parse_stdlib(stdlib_dir, js_name='brython_stdlib.js'):
+    path = os.path.join(stdlib_dir, js_name)
+    with open(path, encoding="utf-8") as fobj:
+        modules = fobj.read()
+        modules = modules[modules.find('{'):
+                          modules.find('__BRYTHON__.update_VFS(')]
+        stdlib = json.loads(modules)
+
+    return stdlib
+
+
+def load_stdlib_sitepackages():
+    """
+    Search brython_stdlib.js, and load it
+    Load site-packages from the current directory
+    :return:
+
+    """
+    stdlib_dir = None
+    for dirname, dirnames, filenames in os.walk(os.getcwd()):
+        for filename in filenames:
+            if filename == "brython_stdlib.js":
+                stdlib_dir = dirname
+                stdlib = parse_stdlib(stdlib_dir)
+                break
+
+    if not stdlib_dir:
+        raise FileNotFoundError("Could not find brython_stdlib.js in this"
+                                " directory or below")
 
-if stdlib_dir is None:
-    raise FileNotFoundError("Could not find brython_stdlib.js in this"
-        " directory or below")
-else:
     # search in site-packages
     sp_dir = os.path.join(stdlib_dir, "Lib", "site-packages")
     if os.path.exists(sp_dir):
@@ -490,6 +536,8 @@ else:
                 if is_package:
                     stdlib[module].append(1)
 
+    return stdlib_dir, stdlib
+
 packages = {os.getcwd(), os.getcwd() + '/Lib/site-packages'}
 
 def is_package(folder):
@@ -508,38 +556,45 @@ def is_package(folder):
             packages.add(folder)
             return True
 
-print('finding packages...')
-for dirname, dirnames, filenames in os.walk(os.getcwd()):
-    for filename in filenames:
-        name, ext = os.path.splitext(filename)
-        if not ext == ".py" or filename == "list_modules.py":
-            continue
-        if dirname == os.getcwd():
-            # modules in the same directory
-            path = os.path.join(dirname, filename)
-            with open(path, encoding="utf-8") as fobj:
-                src = fobj.read()
-            mf = ModulesFinder(dirname)
-            imports = sorted(list(mf.get_imports(src)))
-            user_modules[name] = [ext, src, imports]
-        elif is_package(dirname):
-            # modules in packages below current directory
-            path = os.path.join(dirname, filename)
-            package = dirname[len(os.getcwd()) + 1:].replace(os.sep, '.')
-            if package.startswith('Lib.site-packages.'):
-                package = package[len('Lib.site-packages.'):]
-            if filename == "__init__.py":
-                module_name = package
-            else:
-                module_name = "{}.{}".format(package, name)
-            with open(path, encoding="utf-8") as fobj:
-                src = fobj.read()
-            #mf = ModulesFinder(dirname)
-            #imports = mf.get_imports(src, package or None)
-            #imports = sorted(list(imports))
-            user_modules[module_name] = [ext, src, None]
-            if module_name == package:
-                user_modules[module_name].append(1)
+
+def load_user_modules(module_dir=os.getcwd()):
+    user_modules = {}
+    for dirname, dirnames, filenames in os.walk(module_dir):
+        for filename in filenames:
+            name, ext = os.path.splitext(filename)
+            if not ext == ".py" or filename == "list_modules.py":
+                continue
+            if dirname == os.getcwd():
+                # modules in the same directory
+                path = os.path.join(dirname, filename)
+                with open(path, encoding="utf-8") as fobj:
+                    try:
+                        src = fobj.read()
+                    except:
+                        logger.error("Unable to read %s", path)
+                mf = ModulesFinder(dirname)
+                imports = sorted(list(mf.get_imports(src)))
+                user_modules[name] = [ext, src, imports]
+            elif is_package(dirname):
+                # modules in packages below current directory
+                path = os.path.join(dirname, filename)
+                package = dirname[len(os.getcwd()) + 1:].replace(os.sep, '.')
+                if package.startswith('Lib.site-packages.'):
+                    package = package[len('Lib.site-packages.'):]
+                if filename == "__init__.py":
+                    module_name = package
+                else:
+                    module_name = "{}.{}".format(package, name)
+                with open(path, encoding="utf-8") as fobj:
+                    src = fobj.read()
+                #mf = ModulesFinder(dirname)
+                #imports = mf.get_imports(src, package or None)
+                #imports = sorted(list(imports))
+                user_modules[module_name] = [ext, src, None]
+                if module_name == package:
+                    user_modules[module_name].append(1)
+
+    return user_modules
 
 
 class CharsetDetector(html.parser.HTMLParser):
@@ -644,5 +699,4 @@ class VFSReplacementParser(html.parser.HTMLParser):
 if __name__ == "__main__":
     finder = ModulesFinder()
     finder.inspect()
-    print(sorted(list(finder.modules)))
-
+    # print(sorted(list(finder.modules)))
